use 5.008;
use strict;
use warnings;

package Data::Conveyor::Ticket::Payload::Instruction::Factory;
BEGIN {
  $Data::Conveyor::Ticket::Payload::Instruction::Factory::VERSION = '1.103130';
}
# ABSTRACT: Stage-based conveyor-belt-like ticket handling system

# ptags: DCTPIF
use Error::Hierarchy::Util 'assert_defined';
use parent 'Class::Scaffold::Factory';
use constant INSTRUCTION_CLASS_FOR_NAME =>
  (clear => 'Data::Conveyor::Ticket::Payload::Instruction::Clear',);

sub gen_instruction {
    my ($self, $name, %args) = @_;
    assert_defined $name, 'instruction name';

    # Some instruction classes have a generic structure, they're called
    # Data::Conveyor::Ticket::Payload::Instruction::value_person_organization
    # and such. They're generated in Data::Conveyor::Environment, so they're
    # not in file, so we don't need to load_class() them.
    if (grep { $_ eq $name } $self->delegate->generic_instruction_classes) {
        my $class = $self->delegate->INSTRUCTION_CLASS_BASE() . '::' . $name;
        return $class->new(
            storage_type => $self->storage_type,
            %args
        );
    } else {

        # If it is not a generic value object instruction, go through the
        # normal way of generatic an object.
        return $self->gen_handler(INSTRUCTION_CLASS_FOR_NAME => [$name], %args);
    }
}
1;


__END__
=pod

=head1 NAME

Data::Conveyor::Ticket::Payload::Instruction::Factory - Stage-based conveyor-belt-like ticket handling system

=head1 VERSION

version 1.103130

=head1 METHODS

=head2 gen_instruction

FIXME

=head1 INSTALLATION

See perlmodinstall for information and options on installing Perl modules.

=head1 BUGS AND LIMITATIONS

No bugs have been reported.

Please report any bugs or feature requests through the web interface at
L<http://rt.cpan.org/Public/Dist/Display.html?Name=Data-Conveyor>.

=head1 AVAILABILITY

The latest version of this module is available from the Comprehensive Perl
Archive Network (CPAN). Visit L<http://www.perl.com/CPAN/> to find a CPAN
site near you, or see L<http://search.cpan.org/dist/Data-Conveyor/>.

The development version lives at L<http://github.com/hanekomu/Data-Conveyor>
and may be cloned from L<git://github.com/hanekomu/Data-Conveyor>.
Instead of sending patches, please fork this project using the standard
git and github infrastructure.

=head1 AUTHORS

=over 4

=item *

Marcel Gruenauer <marcel@cpan.org>

=item *

Florian Helmberger <fh@univie.ac.at>

=item *

Achim Adam <ac@univie.ac.at>

=item *

Mark Hofstetter <mh@univie.ac.at>

=item *

Heinz Ekker <ek@univie.ac.at>

=back

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2004 by Marcel Gruenauer.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut

